# pylint: skip-file
""" This module tests the interface module """

import unittest
import mock
import wifiphisher.common.interfaces as interfaces


class TestNetworkAdapter(unittest.TestCase):
    """ Tests NetworkAdapter class """

    def setUp(self):
        """ Set up the tests """

        self.adapter_name = "wlan0"
        self.adapter = interfaces.NetworkAdapter(self.adapter_name)

    def test_get_name(self):
        """ Test get_name method """

        message = "Failed to get correct adapter name!"
        self.assertEqual(self.adapter.get_name(), self.adapter_name, message)

    def test_has_ap_mode_false(self):
        """ Test has_ap_mode method with no AP mode """

        message = "Failed to get False for adapter with no AP support!"
        self.assertFalse(self.adapter.has_ap_mode(), message)

    def test_has_ap_mode_true(self):
        """ Test has_ap_mode method with AP mode available """

        # set AP support to available
        self.adapter.ap_is_supported()

        message = "Failed to get True for adapter with AP support!"
        self.assertTrue(self.adapter.has_ap_mode(), message)

    def test_has_monitor_mode_false(self):
        """ Test has_monitor_mode method with no monitor mode support """

        message = "Failed to get False for adapter with no monitor support!"
        self.assertFalse(self.adapter.has_monitor_mode(), message)

    def test_has_monitor_mode_true(self):
        """ Test has_monitor_mode method with monitor mode available """

        # set monitor support to available
        self.adapter.monitor_is_supported()

        message = "Failed to get True for adapter with monitor support!"
        self.assertTrue(self.adapter.has_monitor_mode(), message)

    def test_is_wireless_false(self):
        """ Test is_wireless method with the default value of False """

        message = "Failed to get False for adapter which is not wireless"
        self.assertFalse(self.adapter.is_wireless(), message)

    def test_is_wireless_true(self):
        """ Test is_wireless method with the default value of test_is_wireless_true """

        # mark interface as wireless
        self.adapter.wireless_is_supported()

        message = "Failed to get True for adapter which is wireless"
        self.assertTrue(self.adapter.is_wireless(), message)


class TestInterfacePropertyDetector(unittest.TestCase):
    """ Test InterfacePropertyDetector class """

    def setUp(self):
        """
        Set up the tests
        """

        self.interface = interfaces.NetworkAdapter("wlan0")
        self.detector = interfaces.InterfacePropertyDetector(self.interface)

    @mock.patch("wifiphisher.common.interfaces.pyw")
    def test_is_interface_wireless_is_wireless_true(self, pyric):
        """
        Test _is_interface_wireless method when interface is wireless
        """

        # steup the return values
        pyric.iswireless.return_value = True

        self.detector._is_interface_wireless()

        message = "Failed to detect interface as wireless when interface is wireless"
        self.assertTrue(self.interface.is_wireless(), message)

    @mock.patch("wifiphisher.common.interfaces.pyw")
    def test_is_interface_wireless_not_wireless_false(self, pyric):
        """
        Test _is_interface_wireless method when interface is
        not wireless
        """

        # steup the return values
        pyric.iswireless.return_value = False

        self.detector._is_interface_wireless()

        message = "Detected the interface as wireless when interface was not wireless"
        self.assertFalse(self.interface.is_wireless(), message)

    @mock.patch("wifiphisher.common.interfaces.pyw")
    def test_find_modes_call_true(self, pyric):
        """
        Test _find_modes method to make sure that it calls pyric
        """

        supported_modes = ["monitor", "AP"]

        # setup the return values
        pyric.getcard.return_value = None
        pyric.devmodes.return_value = supported_modes

        self.detector._find_modes()

        self.assertEqual(self.detector._supported_modes, supported_modes)

    def test_check_monitor_support_has_monitor_true(self):
        """
        Test _check_monitor_support method when monitor mode is
        supported
        """

        # set the supported modes
        self.detector._supported_modes = ["monitor"]

        # call method
        self.detector._check_monitor_support()

        # check the return value
        message = "Failed to detect an interface with monitor support"
        self.assertTrue(self.interface.has_monitor_mode(), message)

    def test_check_monitor_support_no_monitor_false(self):
        """
        Test _check_monitor_support method when monitor mode is
        not supported
        """

        # set the supported modes
        self.detector._supported_modes = ["AP"]

        # call method
        self.detector._check_monitor_support()

        # check the return value
        message = "Detected an interface with monitor support when no monitor support was given"
        self.assertFalse(self.interface.has_monitor_mode(), message)

    def test_check_ap_support_has_ap_true(self):
        """
        Test _check_ap_support method when ap mode is supported
        """

        # set the supported modes
        self.detector._supported_modes = ["AP"]

        # call method
        self.detector._check_ap_support()

        # check the return value
        message = "Failed to detect an interface with ap support"
        self.assertTrue(self.interface.has_ap_mode(), message)

    def test_check_ap_support_no_ap_false(self):
        """
        Test _check_ap_support method when ap mode is not supported
        """

        # set the supported modes
        self.detector._supported_modes = ["monitor"]

        # call method
        self.detector._check_ap_support()

        # check the return value
        message = "Detected an interface with ap support when no ap support was given"
        self.assertFalse(self.interface.has_ap_mode(), message)

    @mock.patch("wifiphisher.common.interfaces.pyw")
    @mock.patch("wifiphisher.common.interfaces.InterfacePropertyDetector")
    @mock.patch("wifiphisher.common.interfaces.InterfacePropertyDetector._find_modes")
    def test_add_supported_properties(self, pyric, ipd, f):
        """
        Test add_supported_properties method to make sure it calls
        the correct methods
        """

        #pyric.getcard.return_value = None
        #pyric.devmodes.return_value = ["monitor"]

        #self._find_modes = mock.MagicMock()
        # TODO stuck here can't find a way to see if any of the methods inside are called.
        # create a detector
        ipd.add_supported_properties()
        f.ssert_called_with()

        #self._find_modes.assert_called_once_with()


class TestNetworkManager(unittest.TestCase):
    """ Tests NetworkManager class """

    def setUp(self):
        """
        Set up the tests
        """

        self.network_manager = interfaces.NetworkManager()

    @mock.patch("wifiphisher.common.interfaces.pyw")
    def test_bring_interface_up_is_down(self, pyric):
        """ Test bring_interface_up method when the interface is down"""

        # setup the return values for pyric
        pyric.getcard.return_value = None
        pyric.isup.return_value = False

        # cratea a interface and call network manager
        interface = interfaces.NetworkAdapter("wlan0")
        self.network_manager.bring_interface_up(interface)

        # check if pyric up method is called
        pyric.up.assert_called_with(None)

    @mock.patch("wifiphisher.common.interfaces.pyw")
    def test_bring_interface_up_is_up(self, pyric):
        """ Test bring_interface_up method when the interface is up"""

        # setup the return values for pyric
        pyric.getcard.return_value = None
        pyric.isup.return_value = True

        # cratea a interface and call network manager
        interface = interfaces.NetworkAdapter("wlan0")
        self.network_manager.bring_interface_up(interface)

        # make sure pyric up method is not called
        pyric.up.assert_not_called()

    @mock.patch("wifiphisher.common.interfaces.pyw")
    def test_bring_interface_down_is_down(self, pyric):
        """ Test bring_interface_up method when the interface is down"""

        # setup the return values for pyric
        pyric.getcard.return_value = None
        pyric.isup.return_value = False

        # cratea a interface and call network manager
        interface = interfaces.NetworkAdapter("wlan0")
        self.network_manager.bring_interface_down(interface)

        # make sure pyric down method is not called
        pyric.down.assert_not_called()

    @mock.patch("wifiphisher.common.interfaces.pyw")
    def test_bring_interface_down_is_up(self, pyric):
        """ Test bring_interface_up method when the interface is up"""

        # setup the return values for pyric
        pyric.getcard.return_value = None
        pyric.isup.return_value = True

        # cratea an interface and call network manager
        interface = interfaces.NetworkAdapter("wlan0")
        self.network_manager.bring_interface_down(interface)

        # make sure pyric down method is called
        pyric.down.assert_called_with(None)

    @mock.patch("wifiphisher.common.interfaces.pyw")
    def test_get_card(self, pyric):
        """ Test _get_card method"""

        # setup the return values for pyric
        pyric.getcard.return_value = None

        # crate an interface anc call network manager
        interface_name = "wlan0"
        interface = interfaces.NetworkAdapter(interface_name)
        self.network_manager._get_card(interface)

        # make sure that get card is called in pyric
        pyric.getcard.assert_called_with(interface_name)

    @mock.patch("wifiphisher.common.interfaces.pyw")
    def test_set_interface_mode_interface(self, pyric):
        """ Test set_interface_mode method"""

        # setup the return values
        pyric.getcard.return_value = None
        pyric.isup.return_value = True

        # crate an interface anc call network manager
        interface_name = "wlan0"
        mode = "monitor"
        interface = interfaces.NetworkAdapter(interface_name)
        self.network_manager.set_interface_mode(interface, mode)

        # make sure that modeset method is called in pyric
        pyric.modeset.assert_called_once_with(None, mode)

    @mock.patch("wifiphisher.common.interfaces.pyw")
    def test_unblock_interface_hard_block(self, pyric):
        """
        Test unblock_interface method when interface is hard blocked
        but not soft blocked
        """

        # setup return values
        pyric.getcard.return_value = None
        pyric.isblocked.return_value = (False, True)

        # crate an interface anc call network manager
        interface_name = "wlan0"
        interface = interfaces.NetworkAdapter(interface_name)

        self.assertRaises(interfaces.InterfaceIsHardBlocked,
                          self.network_manager.unblock_interface, interface)

    @mock.patch("wifiphisher.common.interfaces.pyw")
    def test_unblock_interface_hard_block_soft_block(self, pyric):
        """
        Test unblock_interface method when interface is hard blocked
        and soft blocked
        """

        # setup return values
        pyric.getcard.return_value = None
        pyric.isblocked.return_value = (True, True)

        # crate an interface anc call network manager
        interface_name = "wlan0"
        interface = interfaces.NetworkAdapter(interface_name)

        self.assertRaises(interfaces.InterfaceIsHardBlocked,
                          self.network_manager.unblock_interface, interface)

    @mock.patch("wifiphisher.common.interfaces.pyw")
    def test_unblock_interface_soft_block(self, pyric):
        """
        Test unblock_interface method when interface is soft blocked
        """

        # setup return values
        pyric.getcard.return_value = None
        pyric.isblocked.return_value = (True, False)

        # crate an interface anc call network manager
        interface_name = "wlan0"
        interface = interfaces.NetworkAdapter(interface_name)
        self.network_manager.unblock_interface(interface)

        # make sure that unblock is called in pyric
        pyric.unblock.assert_called_once_with(None)

    @mock.patch("wifiphisher.common.interfaces.pyw")
    def test_unblock_interface_not_block(self, pyric):
        """
        Test unblock_interface method when interface is soft blocked
        """

        # setup return values
        pyric.getcard.return_value = None
        pyric.isblocked.return_value = (False, False)

        # crate an interface anc call network manager
        interface_name = "wlan0"
        interface = interfaces.NetworkAdapter(interface_name)
        self.network_manager.unblock_interface(interface)

        # make sure that unblock is not called
        pyric.unblock.assert_not_called()
